part of 'vehicles_on_the_property.dart';

class VehicleHistoryScreen extends StatefulWidget {
  final VehicleModel model;
  const VehicleHistoryScreen(this.model, {super.key});

  @override
  State<VehicleHistoryScreen> createState() => _VehicleHistoryScreenState();
}

class _VehicleHistoryScreenState extends State<VehicleHistoryScreen>
    with Pageable {
  final apiCall = PhpGroup.getVehiclesByProperty;
  int count = 0;
  final PagingController<int, dynamic> _pagingController =
      PagingController<int, dynamic>(firstPageKey: 1);
  bool _isSnackble = true;

  @override
  void initState() {
    super.initState();
    _pagingController.addPageRequestListener(
      (int pageKey) => fetchPage(
        dataProvider: () async {
          final newItems = await apiCall.call(pageKey.toString());
          if (newItems.jsonBody == null) return (false, null);
          final List<dynamic> vehicles =
              (newItems.jsonBody['vehicles'] as List<dynamic>?) ?? [];

          safeSetState(() {
            count = newItems.jsonBody['total_rows'] ?? 0;
          });

          return (vehicles.isNotEmpty, vehicles);
        },
        onDataUnavailable: (vehicles) {
          setState(() {});
          final bool isFirst = pageKey == 2;
          if (!isFirst && _isSnackble) showNoMoreDataSnackBar(context);

          _pagingController.appendLastPage(vehicles);
        },
        onDataAvailable: (vehicles) {
          setState(() {});
          _pagingController.appendPage(vehicles, pageKey + 1);
        },
        onFetchError: (e, s) {
          DialogUtil.errorDefault(context);
          LogUtil.requestAPIFailed(
              "proccessRequest.php", "", "Consulta de Veículo", e, s);
          setState(() {});
        },
      ),
    );
    _pagingController.addStatusListener(_showError);
  }

  @override
  void dispose() {
    _pagingController.dispose();
    super.dispose();
  }

  Future<void> _showError(PagingStatus status) async {
    if (status == PagingStatus.subsequentPageError) {
      final message = FFLocalizations.of(context).getVariableText(
        enText: 'Something went wrong while fetching a new page.',
        ptText: 'Algo deu errado ao buscar uma nova página.',
      );
      final retry = FFLocalizations.of(context).getVariableText(
        enText: 'Retry',
        ptText: 'Recarregar',
      );

      ScaffoldMessenger.of(context).showSnackBar(
        SnackBar(
          content: Text(message),
          action: SnackBarAction(
            label: retry,
            onPressed: () => _pagingController.retryLastFailedRequest(),
          ),
        ),
      );
    }
  }

  @override
  Widget build(BuildContext context) {
    return Column(
      children: [
        _buildHeader(context),
        _buildBody(context),
      ],
    );
  }

  Widget _buildHeader(BuildContext context) {
    final bodyFontSize = LimitedFontSizeUtil.getBodyFontSize(context);
    return SizedBox(
      height: 30,
      child: Center(
        child: Text(
          (widget.model.amountRegister == '0' ||
                  widget.model.amountRegister == null)
              ? ''
              : "${FFLocalizations.of(context).getVariableText(ptText: "Quantidade de Veículos: ", enText: "Amount of Vehicles: ")}${widget.model.amountRegister}/$count",
          textAlign: TextAlign.right,
          style: TextStyle(
            fontFamily: 'Nunito',
            fontSize: bodyFontSize,
          ),
        ),
      ),
    );
  }

  Expanded _buildBody(BuildContext context) {
    final noDataFound = FFLocalizations.of(context).getVariableText(
      ptText: "Nenhum veículo encontrado!",
      enText: "No vehicle found",
    );
    return buildPaginatedListView<int, dynamic>(
      noDataFound,
      _pagingController,
      _generateItems,
    );
  }

  Widget _generateItems(
    BuildContext context,
    dynamic item,
    int index,
  ) {
    log('item: $item');
    final bool? isOwner = item['isOwnerVehicle'];
    final IconData iconData =
        isOwner == true ? Symbols.garage : Symbols.directions_car;

    final FreCardIcon? cardIcon = isOwner != null
        ? FreCardIcon(
            height: 50,
            width: 100,
            icon: Icon(iconData, size: 80, opticalSize: 10),
          )
        : null;

    final String? tag = item['tag'];
    final bool containTag = tag.isNotNullAndEmpty;
    final Map<String, String> labelsHashMap =
        _generateLabelsHashMap(context, item, tag, containTag);

    final List<Map<String, Color>> statusHashMapList =
        _generateStatusHashMapList(item);

    Future<void> onTapCardItemAction() async {
      await _handleCardItemTap(context, cardIcon, item);
    }

    final statusLinkedHashMap = statusHashMapList
        .map((map) => LinkedHashMap<String, Color>.from(map))
        .toList();
    final length = statusLinkedHashMap.expand((e) => [e.length]);
    final double itemWidthFactor = length == 1 ? 0.25 : 0.50;

    return CardItemTemplateComponentWidget(
      icon: cardIcon,
      labelsHashMap: labelsHashMap,
      statusHashMap: statusHashMapList,
      onTapCardItemAction: onTapCardItemAction,
      itemWidthFactor: itemWidthFactor,
    );
  }

  Map<String, String> _generateLabelsHashMap(
    BuildContext context,
    Map<String, dynamic> item,
    String? tag,
    bool containTag,
  ) {
    final localization = FFLocalizations.of(context);
    return {
      '${localization.getVariableText(ptText: "Placa", enText: "License Plate")}:':
          item['licensePlate'] ?? '',
      '${localization.getVariableText(ptText: "Modelo", enText: "Model")}:':
          item['model'] ?? '',
      '${localization.getVariableText(ptText: "Proprietário", enText: "Owner")}:':
          item['personName'] ?? '',
      if (containTag)
        '${localization.getVariableText(ptText: "Tag", enText: "Tag")}:':
            tag ?? '',
    };
  }

  List<Map<String, Color>> _generateStatusHashMapList(
      Map<String, dynamic> item) {
    final statusHashMap = widget.model.generateStatusColorMap(item, false);
    return statusHashMap != null ? [statusHashMap] : [];
  }

  Future<void> _handleCardItemTap(
    BuildContext context,
    FreCardIcon? cardIcon,
    Map<String, dynamic> item,
  ) async {
    try {
      final dialogContent = widget.model.buildVehicleDetails(
        icon: cardIcon,
        item: item,
        context: context,
        model: widget.model,
      );

      await showDialog<bool>(
        useSafeArea: true,
        context: context,
        builder: (context) => Dialog(
          alignment: Alignment.center,
          child: dialogContent,
        ),
      ) //
          .then((response) async {
        if (response == true) {
          _isSnackble = false;
          _pagingController.refresh();
        } else {
          _isSnackble = true;
        }
      }) //
          .whenComplete(() {});
    } catch (e, s) {
      DialogUtil.errorDefault(context);
      LogUtil.requestAPIFailed(
          "proccessRequest.php", "", "Consulta de Veículos", e, s);
      safeSetState(() {
        // _hasData = false;
        // _loading = false;
      });
    }
  }
}
